@@ -7,6 +7,7 @@ import re, logging, weakref, time, threading, abc
 from .serial_comms import SerialComms
 from .exceptions import CommandError, InvalidStateException
 from gsmmodem.exceptions import TimeoutException
+import compat # For Python 2.6 compatibility
 
 class GsmModem(SerialComms):
     """ Main class for interacting with an attached GSM modem """
@@ -24,11 +25,9 @@ class GsmModem(SerialComms):
     # Used for parsing SMS message reads
     CMGR_SM_DELIVER_REGEX = re.compile(r'^\+CMGR:\s*"([^"]+)","([^"]+)",[^,]*,"([^"]+)"$')
     # Used for parsing USSD event notifications
-    CUSD_REGEX = re.compile(r'^\+CUSD:\s*(\d),"(.*)",(\d+)$')
+    CUSD_REGEX = re.compile(r'^\+CUSD:\s*(\d),"(.*)",(\d+)$')    
     
-    
-    
-    def __init__(self, port, baudrate=9600, incomingCallCallbackFunc=None, smsReceivedCallbackFunc=None):
+    def __init__(self, port, baudrate=115200, incomingCallCallbackFunc=None, smsReceivedCallbackFunc=None):
         super(GsmModem, self).__init__(port, baudrate, notifyCallbackFunc=self._handleModemNotification)
         self.incomingCallCallback = incomingCallCallbackFunc or self._placeholderCallback
         self.smsReceivedCallback = smsReceivedCallbackFunc or self._placeholderCallback
@@ -36,29 +35,77 @@ class GsmModem(SerialComms):
         self._callingLineIdentification = False
         # Flag indicating whether incoming call notifications have extended information
         self._extendedIncomingCallIndication = False
-        # Dict containing current active calls (ringing and/or answered)
-        self.activeCalls = {}#weakref.WeakValueDictionary()
+        # Current active calls (ringing and/or answered), key is the unique call ID (not the remote number)
+        self.activeCalls = {}
         # Dict containing sent SMS messages (to track their delivery status)
         self.sentSms = {}
         self._ussdSessionEvent = None # threading.Event
         self._ussdResponse = None # gsmmodem.modem.Ussd
+        self._dialEvent = None # threading.Event
+        self._dialResponse = None # gsmmodem.modem.Call
+        self._waitForAtdResponse = True # Flag that controls if we should wait for an immediate response to ATD, or not
+        self.callStatusUpdates = [] # populated during connect() - contains regexes and handlers for detecting/handling call status updates    
         
     def connect(self, runInit=True):
         """ Opens the port and initializes the modem """
-        self.log.debug('Connecting to modem on port {} at {}bps'.format(self.port, self.baudrate))
+        self.log.debug('Connecting to modem on port {0} at {1}bps'.format(self.port, self.baudrate))
         super(GsmModem, self).connect()                
         # Send some initialization commands to the modem
         self.write('ATZ') # reset configuration
         self.write('ATE0') # echo off
         self.write('AT+CFUN=1') # enable full modem functionality
         self.write('AT+CMEE=1') # enable detailed error messages
-        # disable misc notifications (we will enable what we need in a bit) - not all modems support this command; doesn't matter
-        self.write('AT+WIND=0', parseError=False)        
         
+        # Get list of supported commands from modem
+        commands = self.supportedCommands
+
+        # Device-specific settings
+        if commands != None:
+            if '^CVOICE' in commands:
+                self.write('AT^CVOICE=0', parseError=False) # Enable voice calls
+            if '+VTS' in commands: # Check for DTMF sending support
+                Call.dtmfSupport = True
+            elif '^DTMF' in commands:
+                # Huawei modems use ^DTMF to send DTMF tones; use that instead
+                Call.DTMF_COMMAND_BASE = '^DTMF=1,'
+                Call.dtmfSupport = True
+            
+        # Enable general notifications on Wavecom-like devices - also use result to select the type of call status updates we will receive
+        try:
+            self.write('AT+WIND=63')
+        except CommandError:
+            # Modem does not support +WIND notifications, use Hauwei's ^NOTIFICATIONs
+            self.log.info('Loading Huawei call update table')
+            self.callStatusUpdates = ((re.compile(r'^\^ORIG:(\d),(\d)$'), self._handleCallInitiated),
+                                      (re.compile(r'^\^CONN:(\d),(\d)$'), self._handleCallAnswered),
+                                      (re.compile(r'^\^CEND:(\d),(\d),(\d)+,(\d)+$'), self._handleCallEnded))            
+            self._waitForAtdResponse = True # Huawei modems return OK immediately after issuing ATD            
+        else:
+            # +WIND notifications supported
+            self.log.info('Loading Wavecom call update table')
+            self.callStatusUpdates = ((re.compile(r'^\+WIND: 5,(\d)$'), self._handleCallInitiated),
+                                      (re.compile(r'^OK$'), self._handleCallAnswered),
+                                      (re.compile(r'^\+WIND: 6,(\d)$'), self._handleCallEnded))            
+            self._waitForAtdResponse = False # Wavecom modems return OK only when the call is answered
+            if commands == None: # older modem, assume it has standard DTMF support
+                Call.dtmfSupport = True
+        
+        # General meta-information setup
+        self.write('AT+COPS=0,0', parseError=False) # Automatic network selection, use long alphanumeric name format
+                
         # SMS setup
         self.write('AT+CMGF=1') # Switch to text mode for SMS messages
         self.write('AT+CSMP=49,167') # Enable delivery reports
-        self.write('AT+CPMS="SM","SM","SR"') # Set message storage
+        
+        # Set message storage, but first check what the modem supports - example response: +CPMS: (("SM","BM","SR"),("SM"))
+        cpmsSupport = self.write('AT+CPMS=?')[0].split(' ', 1)[1].split('),(')        
+        cpmsItems = ['"SM"', '"SM"', '"SR"'][:len(cpmsSupport)]
+        #cpmsItems = cpmsItems[:len(cpmsSupport)]
+        for i in xrange(len(cpmsItems)):            
+            if cpmsItems[i] not in cpmsSupport[i]:
+                cpmsItems[i] = ''
+        self.write('AT+CPMS={0}'.format(','.join(cpmsItems))) # Set message storage        
+        #self.write('AT+CPMS="SM","SM","SR"') # Set message storage
         self.write('AT+CNMI=2,1,0,2') # Set message notifications
         
         # Incoming call notification setup        
@@ -66,19 +113,20 @@ class GsmModem(SerialComms):
             self.write('AT+CLIP=1') # Enable calling line identification presentation
         except CommandError, clipError:
             self._callingLineIdentification = False
-            self.log.warn('Incoming call calling line identification (caller ID) not supported by modem. Error: {}'.format(clipError))
+            self.log.warn('Incoming call calling line identification (caller ID) not supported by modem. Error: {0}'.format(clipError))
         else:
             self._callingLineIdentification = True
             try:
                 self.write('AT+CRC=1') # Enable extended format of incoming indication (optional)
             except CommandError, crcError:
                 self._extendedIncomingCallIndication = False
-                self.log.info('Extended format incoming call indication not supported by modem. Error: {}'.format(crcError))
+                self.log.info('Extended format incoming call indication not supported by modem. Error: {0}'.format(crcError))
             else:
                 self._extendedIncomingCallIndication = True        
 
         # Call control setup
         self.write('AT+CVHU=0') # Enable call hang-up with ATH command
+        
 
     def write(self, data, waitForResponse=True, timeout=5, parseError=True, writeTerm='\r', expectedResponseTermSeq=None):
         """ Write data to the modem
@@ -110,7 +158,7 @@ class GsmModem(SerialComms):
 
     @property
     def signalStrength(self):
-        """ @return The network signal strength as an integer between 0 and 99, or -1 if it is unknown """
+        """ @return: The network signal strength as an integer between 0 and 99, or -1 if it is unknown """
         csq = self.CSQ_REGEX.match(self.write('AT+CSQ')[0])
         if csq:
             ss = int(csq.group(1))
@@ -118,6 +166,50 @@ class GsmModem(SerialComms):
         else:
             raise CommandError()
 
+    @property
+    def manufacturer(self):
+        """ @return: The modem's manufacturer's name """
+        return self.write('AT+CGMI')[0]
+    
+    @property
+    def model(self):
+        """ @return: The modem's model name """
+        return self.write('AT+CGMM')[0]
+    
+    @property
+    def revision(self):
+        """ @return: The modem's software revision, or None if not known/supported """
+        try:
+            return self.write('AT+CGMR')[0]
+        except CommandError:
+            return None
+    
+    @property
+    def imei(self):
+        """ @return: The modem's serial number (IMEI number) """
+        return self.write('AT+CGSN')[0]
+    
+    @property
+    def imsi(self):
+        """ @return: The IMSI (International Mobile Subscriber Identity) of the SIM card. The PIN may need to be entered before reading the IMSI """
+        return self.write('AT+CIMI')[0]
+    
+    @property
+    def networkName(self):
+        """ @return: the name of the GSM Network Operator to which the modem is connected """
+        response = self.write('AT+COPS?') # response format: +COPS: mode,format,"operator_name",x
+        copsMatch = re.match(r'^\+COPS: (\d),(\d),"(.+)",\d$', response)
+        if copsMatch:
+            return copsMatch.group(3)
+
+    @property
+    def supportedCommands(self):
+        """ @return: list of AT commands supported by this modem (without the AT prefix). Returns None if not known """
+        try:
+            return self.write('AT+CLAC')[0][6:].split(',') # remove the +CLAC: prefix before splitting
+        except CommandError:
+            return None
+
     def waitForNetworkCoverage(self, timeout=None):
         """ Block until the modem has GSM network coverage.
         
@@ -164,7 +256,7 @@ class GsmModem(SerialComms):
         sends the specified string in the existing USSD session (if any)
                 
         @param ussdString: The USSD access number to dial
-        @param timeout: Maximum time to wait a response, in seconds
+        @param responseTimeout: Maximum time to wait a response, in seconds
         
         @raise TimeoutException: if no response is received in time
         
@@ -179,7 +271,26 @@ class GsmModem(SerialComms):
             return self._ussdResponse
         else: # Response timed out
             self._ussdSessionEvent = None            
-            raise TimeoutException()        
+            raise TimeoutException()
+    
+    def dial(self, number, timeout=5):
+        """ Calls the specified phone number using a voice phone call
+        
+        @param number: The phone number to dial
+        @param timeout: Maximum time to wait for the call to be established
+        """
+        self.write('ATD{0};'.format(number), waitForResponse=self._waitForAtdResponse)
+        # Wait for the ^ORIG notification message
+        self._dialEvent = threading.Event()
+        if self._dialEvent.wait(timeout):
+            self._dialEvent = None
+            callId, callType = self._dialResponse
+            call = Call(self, callId, callType, number)
+            self.activeCalls[callId] = call
+            return call
+        else: # Call establishing timed out
+            self._dialEvent = None            
+            raise TimeoutException()
 
     def _handleModemNotification(self, lines):
         """ Handler for unsolicited notifications from the modem
@@ -195,7 +306,7 @@ class GsmModem(SerialComms):
         """ Implementation of _handleModemNotification() to be run in a separate thread 
         
         @param lines The lines that were read
-        """
+        """        
         firstLine = lines[0]
         if 'RING' in firstLine:
             # Incoming call (or existing call is ringing)
@@ -205,8 +316,16 @@ class GsmModem(SerialComms):
             self._handleSmsReceived(firstLine)
         elif firstLine.startswith('+CUSD'):
             # USSD notification - either a response or a MT-USSD ("push USSD") message
-            self._handleUssd(firstLine) 
+            self._handleUssd(firstLine)
         else:
+            # Check for call status updates
+            for updateRegex, handlerFunc in self.callStatusUpdates:
+                match = updateRegex.match(firstLine)
+                if match:
+                    # Handle the update
+                    handlerFunc(match)
+                    return
+            # If this is reached, the notification wasn't handled
             self.log.debug('Unhandled unsolicited modem notification:', lines)
     
     def _handleIncomingCall(self, lines):
@@ -228,15 +347,54 @@ class GsmModem(SerialComms):
                 callerNumber = ton = callerName = None
         else:
             callerNumber = ton = callerName = None
-            
-        if callerNumber in self.activeCalls:
-            call = self.activeCalls[callerNumber]
-            call.ringCount += 1
-        else:        
-            call = IncomingCall(self, callerNumber, ton, callerName, callType)
-            self.activeCalls[callerNumber] = call        
-        self.incomingCallCallback(call)
         
+        call = None
+        for activeCall in self.activeCalls.itervalues():
+            if activeCall.number == callerNumber:
+                call = self.activeCall
+                call.ringCount += 1
+        if call == None:
+            callId = len(self.activeCalls) + 1;
+            call = IncomingCall(self, callerNumber, ton, callerName, callId, callType)
+            self.activeCalls[callId] = call
+        self.incomingCallCallback(call)
+    
+    def _handleCallInitiated(self, regexMatch):
+        """ Handler for "outgoing call initiated" event notification line """
+        if self._dialEvent:
+            #origMatch = re.match(r'^\^ORIG:(\d),(\d)$', notificationLine)
+            #if origMatch:
+                # Return callId, callType
+            groups = regexMatch.groups()
+            if len(groups) >= 2:                                
+                self._dialResponse = (int(groups[0]) , int(groups[1]))
+            else:
+                self._dialResponse = (int(groups[0]), 1) # assume call type: VOICE
+            self._dialEvent.set()
+                
+    def _handleCallAnswered(self, regexMatch):
+        """ Handler for "outgoing call answered" event notification line """
+        #connMatch = re.match(r'^\^CONN:(\d),(\d)$', notificationLine)
+        #if connMatch:
+        groups = regexMatch.groups()
+        if len(groups) > 1:
+            callId = int(groups[0])
+            self.activeCalls[callId].answered = True
+        else:
+            # Call ID not available for this notificaiton - check for the first outgoing call that has not been answered
+            for call in self.activeCalls.itervalues():
+                if call.answered == False and type(call) == Call:
+                    call.answered = True
+                    return
+    
+    def _handleCallEnded(self, regexMatch):
+        #cendMatch = re.match(r'^\^CEND:(\d),(\d),(\d)+,(\d)+$', notificationLine)
+        #if cendMatch:
+        callId = int(regexMatch.group(1))
+        if callId in self.activeCalls:
+            self.activeCalls[callId].answered = False
+            del self.activeCalls[callId]
+    
     def _handleSmsReceived(self, notificationLine):
         """ Handler for "new SMS" unsolicited notification line """
         cmtiMatch = self.CMTI_REGEX.match(notificationLine)
@@ -247,7 +405,7 @@ class GsmModem(SerialComms):
             self.smsReceivedCallback(sms)            
     
     def _readStoredSmsMessage(self, msgIndex):
-        msgData = self.write('AT+CMGR={}'.format(msgIndex))
+        msgData = self.write('AT+CMGR={0}'.format(msgIndex))
         # Parse meta information
         cmgrMatch = self.CMGR_SM_DELIVER_REGEX.match(msgData[0])
         if not cmgrMatch:
@@ -258,7 +416,7 @@ class GsmModem(SerialComms):
         return ReceivedSms(self, msgStatus, number, msgTime, msgText)
             
     def _deleteStoredMessage(self, msgIndex):
-        self.write('AT+CMGD={}'.format(msgIndex))
+        self.write('AT+CMGD={0}'.format(msgIndex))
     
     def _handleUssd(self, notificationLine):
         """ Handler for USSD event notification line """
@@ -272,29 +430,76 @@ class GsmModem(SerialComms):
     
     def _placeHolderCallback(self, *args):
         """ Does nothing """
-        self.log.debug('called with args: {}'.format(args))
+        self.log.debug('called with args: {0}'.format(args))
 
 
-class IncomingCall(object):
+class Call(object):
+    """ A voice call """
+    
+    DTMF_COMMAND_BASE = '+VTS='    
+    dtmfSupport = False # Indicates whether or not DTMF tones can be sent in calls
+    
+    def __init__(self, gsmModem, callId, callType, number):
+        """
+        @param gsmModem: GsmModem instance that created this object
+        @param number: The number that is being called        
+        """
+        self._gsmModem = weakref.proxy(gsmModem)
+        # Unique ID of this call
+        self.id = callId
+        # Call type (VOICE == 0, etc)
+        self.type = callType        
+        # The remote number of this call (destination or origin)
+        self.number = number                
+        # Flag indicating whether the call has been answered or not
+        self.answered = False        
+    
+    def sendDtmfTone(self, tones):
+        """ Send a DTMF tone to the remote party (only allowed for an answered call) 
+        
+        Note: this is highly device-dependent, and might not work
+        
+        @param digits: A str containining one or more DTMF tones to play, e.g. "3" or "*123#"
+
+        @raise CommandError: if the command failed/is not supported
+        """        
+        if self.answered:
+            if len(tones) > 1:
+                cmd = ('AT{0}{1};{0}' + ';{0}'.join(tones[1:])).format(self.DTMF_COMMAND_BASE, tones[0])                
+            else:
+                cmd = 'AT+VTS={0}'.format(tones)            
+            self._gsmModem.write(cmd)
+        else:
+            raise InvalidStateException('Call is not active (it has not yet been answered, or it has ended).')
+    
+    def hangup(self):
+        """ End the phone call. """
+        self._gsmModem.write('ATH')
+        self.answered = False
+        if self.id in self._gsmModem.activeCalls:
+            del self._gsmModem.activeCalls[self.id]
+
+
+class IncomingCall(Call):
+    
+    CALL_TYPE_MAP = {'VOICE': 0}
+    
     """ Represents an incoming call, conveniently allowing access to call meta information and -control """     
-    def __init__(self, gsmModem, number, ton, callerName, callType):
+    def __init__(self, gsmModem, number, ton, callerName, callId, callType):
         """
         @param gsmModem: GsmModem instance that created this object
         @param number: Caller number
         @param ton: TON (type of number/address) in integer format
         @param callType: Type of the incoming call (VOICE, FAX, DATA, etc)
         """
-        self._gsmModem = weakref.proxy(gsmModem)
-        # The number that is dialling
-        self.number = number
+        if type(callType) == str:
+            callType = self.CALL_TYPE_MAP[callType] 
+        super(IncomingCall, self).__init__(gsmModem, callId, callType, number)        
         # Type attribute of the incoming call
         self.ton = ton
-        self.callerName = callerName
-        self.type = callType
+        self.callerName = callerName        
         # Flag indicating whether the call is ringing or not
-        self.ringing = True
-        # Flag indicating whether the call has been answered or not
-        self.answered = False
+        self.ringing = True        
         # Amount of times this call has rung (before answer/hangup)
         self.ringCount = 1
     
@@ -306,35 +511,12 @@ class IncomingCall(object):
             self._gsmModem.write('ATA')
             self.ringing = False
             self.answered = True
-        return self
-    
-    def sendDtmfTone(self, tones):
-        """ Send a DTMF tone to the remote party (only allowed for an answered call) 
-        
-        Note: this is highly device-dependent, and might not work
-        
-        @param digits: A str containining one or more DTMF tones to play, e.g. "3" or "*123#"
-
-        @raise CommandError: if the command failed/is not supported
-        """
-        if self.answered:
-            if len(tones) > 1:
-                cmd = 'AT+VTS={}'.format(';+VTS='.join(tones))
-            else:
-                cmd = 'AT+VTS={}'.format(tones)            
-            self._gsmModem.write(cmd)
-        else:
-            raise InvalidStateException('Call is not active (it has not yet been answered, or it has ended).')
+        return self    
 
     def hangup(self):
-        """ End the phone call.        
-        @return: self (for chaining method calls)
-        """
-        self._gsmModem.write('ATH')
+        """ End the phone call. """
         self.ringing = False
-        self.answered = False
-        if self.number in self._gsmModem.activeCalls:
-            del self._gsmModem.activeCalls[self.number]
+        super(IncomingCall, self).hangup()
 
 
 class Sms(object):
