@@ -9,7 +9,7 @@ import re
 import serial # pyserial: http://pyserial.sourceforge.net
 
 from .exceptions import TimeoutException
-
+import compat # For Python 2.6 compatibility
 
 class SerialComms(object):
     """ Wraps all low-level serial communications (actual read/write operations) """
@@ -21,10 +21,10 @@ class SerialComms(object):
     # Default timeout for serial port reads (in seconds)
     timeout = 1
         
-    def __init__(self, port, baudrate=9600, notifyCallbackFunc=None, *args, **kwargs):        
+    def __init__(self, port, baudrate=115200, notifyCallbackFunc=None, *args, **kwargs):        
         self.alive = False
         self.port = port
-        self.baudrate = 9600
+        self.baudrate = baudrate
         
         self._responseEvent = None # threading.Event()
         self._expectResponseTermSeq = None # expected response terminator sequence
@@ -57,14 +57,17 @@ class SerialComms(object):
             self._response.append(line)
             if not checkForResponseTerm or self.RESPONSE_TERM.match(line):
                 # End of response reached; notify waiting thread
+                #print 'response:', self._response
                 self._responseEvent.set()
         else:
             # Nothing was waiting for this - treat it as a notification
             self._notification.append(line)
             if self.serial.inWaiting() == 0:
                 # No more chars on the way for this notification - notify higher-level callback
+                #print 'notification:', self._notification
                 self.notifyCallback(self._notification)
                 self._notification = []
+                
 
     def _placeholderCallback(self, *args, **kwargs):
         """ Placeholder callback function (does nothing) """
@@ -81,7 +84,7 @@ class SerialComms(object):
             while self.alive:
                 data = self.serial.read(1)
                 if data != '': # check for timeout
-                    #print(' RX:', data,'({})'.format(ord(data)))
+                    #print(' RX:', data,'({0})'.format(ord(data)))
                     rxBuffer.append(data)
                     if rxBuffer[-readTermLen:] == readTermSeq:                        
                         # A line (or other logical segment) has been read
